package java.awt;

import java.awt.event.FocusListener;
import java.util.Locale;
import javax.accessibility.AccessibleRole;
import javax.accessibility.AccessibleState;
import javax.accessibility.AccessibleStateSet;
import javax.accessibility.AccessibleSelection;
import javax.accessibility.AccessibleContext;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.util.EventListener;
import java.awt.peer.ListPeer;
import java.awt.event.ItemListener;
import java.awt.event.ActionListener;
import java.util.Vector;
import javax.accessibility.Accessible;

public class List extends Component implements ItemSelectable, Accessible
{
    Vector<String> items;
    int rows;
    boolean multipleMode;
    int[] selected;
    int visibleIndex;
    transient ActionListener actionListener;
    transient ItemListener itemListener;
    private static final String base = "list";
    private static int nameCounter;
    private static final long serialVersionUID = -3304312411574666869L;
    static final int DEFAULT_VISIBLE_ROWS = 4;
    private int listSerializedDataVersion;
    
    public List() throws HeadlessException {
        this(0, false);
    }
    
    public List(int n) throws HeadlessException {
        this(n, false);
    }
    
    public List(int n, boolean multipleMode) throws HeadlessException {
        this.items = new Vector();
        this.rows = 0;
        this.multipleMode = false;
        this.selected = new int[0];
        this.visibleIndex = -1;
        this.listSerializedDataVersion = 1;
        GraphicsEnvironment.checkHeadless();
        this.rows = ((n != 0) ? n : 4);
        this.multipleMode = multipleMode;
    }
    
    @Override
    String constructComponentName() {
        synchronized (List.class) {
            return "list" + List.nameCounter++;
        }
    }
    
    @Override
    public void addNotify() {
        synchronized (this.getTreeLock()) {
            if (this.peer == null) {
                this.peer = this.getToolkit().createList(this);
            }
            super.addNotify();
        }
    }
    
    @Override
    public void removeNotify() {
        synchronized (this.getTreeLock()) {
            ListPeer listPeer = (ListPeer)this.peer;
            if (listPeer != null) {
                this.selected = listPeer.getSelectedIndexes();
            }
            super.removeNotify();
        }
    }
    
    public int getItemCount() {
        return this.countItems();
    }
    
    @Deprecated
    public int countItems() {
        return this.items.size();
    }
    
    public String getItem(int n) {
        return this.getItemImpl(n);
    }
    
    final String getItemImpl(int index) {
        return (String)this.items.elementAt(index);
    }
    
    public synchronized String[] getItems() {
        String[] anArray = new String[this.items.size()];
        this.items.copyInto(anArray);
        return anArray;
    }
    
    public void add(String s) {
        this.addItem(s);
    }
    
    @Deprecated
    public void addItem(String s) {
        this.addItem(s, -1);
    }
    
    public void add(String s, int n) {
        this.addItem(s, n);
    }
    
    @Deprecated
    public synchronized void addItem(String s, int index) {
        if (index < -1 || index >= this.items.size()) {
            index = -1;
        }
        if (s == null) {
            s = "";
        }
        if (index == -1) {
            this.items.addElement(s);
        }
        else {
            this.items.insertElementAt(s, index);
        }
        ListPeer listPeer = (ListPeer)this.peer;
        if (listPeer != null) {
            listPeer.add(s, index);
        }
    }
    
    public synchronized void replaceItem(String s, int n) {
        this.remove(n);
        this.add(s, n);
    }
    
    public void removeAll() {
        this.clear();
    }
    
    @Deprecated
    public synchronized void clear() {
        ListPeer listPeer = (ListPeer)this.peer;
        if (listPeer != null) {
            listPeer.removeAll();
        }
        this.items = new Vector();
        this.selected = new int[0];
    }
    
    public synchronized void remove(String s) {
        int index = this.items.indexOf(s);
        if (index < 0) {
            throw new IllegalArgumentException("item " + s + " not found in list");
        }
        this.remove(index);
        return;
    }
    
    public void remove(int n) {
        this.delItem(n);
    }
    
    @Deprecated
    public void delItem(int n) {
        this.delItems(n, n);
    }
    
    public synchronized int getSelectedIndex() {
        int[] selectedIndexes = this.getSelectedIndexes();
        return (selectedIndexes.length == 1) ? selectedIndexes[0] : -1;
    }
    
    public synchronized int[] getSelectedIndexes() {
        ListPeer listPeer = (ListPeer)this.peer;
        if (listPeer != null) {
            this.selected = listPeer.getSelectedIndexes();
        }
        return (int[])this.selected.clone();
    }
    
    public synchronized String getSelectedItem() {
        int selectedIndex = this.getSelectedIndex();
        return (selectedIndex < 0) ? null : this.getItem(selectedIndex);
    }
    
    public synchronized String[] getSelectedItems() {
        int[] selectedIndexes = this.getSelectedIndexes();
        String[] array = new String[selectedIndexes.length];
        for (int i = 0; i < selectedIndexes.length; ++i) {
            array[i] = this.getItem(selectedIndexes[i]);
        }
        return array;
    }
    
    @Override
    public Object[] getSelectedObjects() {
        return this.getSelectedItems();
    }
    
    public void select(int n) {
        ListPeer listPeer;
        do {
            listPeer = (ListPeer)this.peer;
            if (listPeer != null) {
                listPeer.select(n);
                return;
            }
            synchronized (this) {
                boolean n2 = false;
                int i = 0;
                while (i < this.selected.length) {
                    if (this.selected[i] == n) {
                        n2 = true;
                        break;
                    }
                    ++i;
                }
                if (n2 == 0) {
                    if (!this.multipleMode) {
                        (this.selected = new int[1])[0] = n;
                    }
                    else {
                        int[] selected = new int[this.selected.length + 1];
                        System.arraycopy(this.selected, 0, selected, 0, this.selected.length);
                        selected[this.selected.length] = n;
                        this.selected = selected;
                    }
                }
                else {
                    continue;
                }
            }
        } while (listPeer != this.peer);
    }
    
    public synchronized void deselect(int n) {
        ListPeer listPeer = (ListPeer)this.peer;
        if (listPeer != null && (this.isMultipleMode() || this.getSelectedIndex() == n)) {
            listPeer.deselect(n);
        }
        int i = 0;
        while (i < this.selected.length) {
            if (this.selected[i] == n) {
                int[] selected = new int[this.selected.length - 1];
                System.arraycopy(this.selected, 0, selected, 0, i);
                System.arraycopy(this.selected, i + 1, selected, i, this.selected.length - (i + 1));
                this.selected = selected;
                return;
            }
            ++i;
        }
    }
    
    public boolean isIndexSelected(int n) {
        return this.isSelected(n);
    }
    
    @Deprecated
    public boolean isSelected(int n) {
        int[] selectedIndexes = this.getSelectedIndexes();
        int i = 0;
        while (i < selectedIndexes.length) {
            if (selectedIndexes[i] == n) {
                return true;
            }
            ++i;
        }
        return false;
    }
    
    public int getRows() {
        return this.rows;
    }
    
    public boolean isMultipleMode() {
        return this.allowsMultipleSelections();
    }
    
    @Deprecated
    public boolean allowsMultipleSelections() {
        return this.multipleMode;
    }
    
    public void setMultipleMode(boolean multipleSelections) {
        this.setMultipleSelections(multipleSelections);
    }
    
    @Deprecated
    public synchronized void setMultipleSelections(boolean b) {
        if (b != this.multipleMode) {
            this.multipleMode = b;
            ListPeer listPeer = (ListPeer)this.peer;
            if (listPeer != null) {
                listPeer.setMultipleMode(b);
            }
        }
    }
    
    public int getVisibleIndex() {
        return this.visibleIndex;
    }
    
    public synchronized void makeVisible(int visibleIndex) {
        this.visibleIndex = visibleIndex;
        ListPeer listPeer = (ListPeer)this.peer;
        if (listPeer != null) {
            listPeer.makeVisible(visibleIndex);
        }
    }
    
    public Dimension getPreferredSize(int n) {
        return this.preferredSize(n);
    }
    
    @Deprecated
    public Dimension preferredSize(int n) {
        synchronized (this.getTreeLock()) {
            ListPeer listPeer = (ListPeer)this.peer;
            return (listPeer != null) ? listPeer.getPreferredSize(n) : super.preferredSize();
        }
    }
    
    @Override
    public Dimension getPreferredSize() {
        return this.preferredSize();
    }
    
    @Deprecated
    @Override
    public Dimension preferredSize() {
        synchronized (this.getTreeLock()) {
            return (this.rows > 0) ? this.preferredSize(this.rows) : super.preferredSize();
        }
    }
    
    public Dimension getMinimumSize(int n) {
        return this.minimumSize(n);
    }
    
    @Deprecated
    public Dimension minimumSize(int n) {
        synchronized (this.getTreeLock()) {
            ListPeer listPeer = (ListPeer)this.peer;
            return (listPeer != null) ? listPeer.getMinimumSize(n) : super.minimumSize();
        }
    }
    
    @Override
    public Dimension getMinimumSize() {
        return this.minimumSize();
    }
    
    @Deprecated
    @Override
    public Dimension minimumSize() {
        synchronized (this.getTreeLock()) {
            return (this.rows > 0) ? this.minimumSize(this.rows) : super.minimumSize();
        }
    }
    
    @Override
    public synchronized void addItemListener(ItemListener itemListener) {
        if (itemListener == null) {
            return;
        }
        this.itemListener = AWTEventMulticaster.add(this.itemListener, itemListener);
        this.newEventsOnly = true;
        return;
    }
    
    @Override
    public synchronized void removeItemListener(ItemListener itemListener) {
        if (itemListener == null) {
            return;
        }
        this.itemListener = AWTEventMulticaster.remove(this.itemListener, itemListener);
        return;
    }
    
    public synchronized ItemListener[] getItemListeners() {
        return (ItemListener[])this.getListeners(ItemListener.class);
    }
    
    public synchronized void addActionListener(ActionListener actionListener) {
        if (actionListener == null) {
            return;
        }
        this.actionListener = AWTEventMulticaster.add(this.actionListener, actionListener);
        this.newEventsOnly = true;
        return;
    }
    
    public synchronized void removeActionListener(ActionListener actionListener) {
        if (actionListener == null) {
            return;
        }
        this.actionListener = AWTEventMulticaster.remove(this.actionListener, actionListener);
        return;
    }
    
    public synchronized ActionListener[] getActionListeners() {
        return (ActionListener[])this.getListeners(ActionListener.class);
    }
    
    @Override
    public <T extends EventListener> T[] getListeners(Class<T> clazz) {
        ActionListener actionListener;
        if (clazz == ActionListener.class) {
            actionListener = this.actionListener;
        }
        else if (clazz == ItemListener.class) {
            actionListener = this.itemListener;
        }
        else {
            return super.getListeners(clazz);
        }
        return AWTEventMulticaster.getListeners(actionListener, clazz);
    }
    
    @Override
    boolean eventEnabled(AWTEvent awtEvent) {
        switch (awtEvent.id) {
            case 1001: {
                return (this.eventMask & 0x80L) != 0x0L || this.actionListener != null;
            }
            case 701: {
                return (this.eventMask & 0x200L) != 0x0L || this.itemListener != null;
            }
            default: {
                return super.eventEnabled(awtEvent);
            }
        }
    }
    
    @Override
    protected void processEvent(AWTEvent awtEvent) {
        if ((awtEvent instanceof ItemEvent)) {
            this.processItemEvent((ItemEvent)awtEvent);
            return;
        }
        if ((awtEvent instanceof ActionEvent)) {
            this.processActionEvent((ActionEvent)awtEvent);
            return;
        }
        super.processEvent(awtEvent);
        return;
    }
    
    protected void processItemEvent(ItemEvent itemEvent) {
        ItemListener itemListener = this.itemListener;
        if (itemListener != null) {
            itemListener.itemStateChanged(itemEvent);
        }
    }
    
    protected void processActionEvent(ActionEvent actionEvent) {
        ActionListener actionListener = this.actionListener;
        if (actionListener != null) {
            actionListener.actionPerformed(actionEvent);
        }
    }
    
    @Override
    protected String paramString() {
        return super.paramString() + ",selected=" + this.getSelectedItem();
    }
    
    @Deprecated
    public synchronized void delItems(int n, int i) {
        for (; i >= n; --i) {
            this.items.removeElementAt(i);
        }
        ListPeer listPeer = (ListPeer)this.peer;
        if (listPeer != null) {
            listPeer.delItems(n, i);
        }
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        synchronized (this) {
            ListPeer listPeer = (ListPeer)this.peer;
            if (listPeer != null) {
                this.selected = listPeer.getSelectedIndexes();
            }
        }
        objectOutputStream.defaultWriteObject();
        AWTEventMulticaster.save(objectOutputStream, "itemL", this.itemListener);
        AWTEventMulticaster.save(objectOutputStream, "actionL", this.actionListener);
        objectOutputStream.writeObject(null);
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws ClassNotFoundException, IOException, HeadlessException {
        GraphicsEnvironment.checkHeadless();
        objectInputStream.defaultReadObject();
        Object object;
        while (null != (object = objectInputStream.readObject())) {
            String intern = ((String)object).intern();
            if ("itemL" == intern) {
                this.addItemListener((ItemListener)(ItemListener)objectInputStream.readObject());
            }
            else if ("actionL" == intern) {
                this.addActionListener((ActionListener)(ActionListener)objectInputStream.readObject());
            }
            else {
                objectInputStream.readObject();
            }
        }
    }
    
    @Override
    public AccessibleContext getAccessibleContext() {
        if (this.accessibleContext == null) {
            this.accessibleContext = new List.AccessibleAWTList();
        }
        return this.accessibleContext;
    }
    
    static {
        List.nameCounter = 0;
    }
    
    protected class AccessibleAWTList extends Component.AccessibleAWTComponent implements AccessibleSelection, ItemListener, ActionListener
    {
        private static final long serialVersionUID = 7924617370136012829L;
        
        public AccessibleAWTList() {
            List.this.addActionListener(this);
            List.this.addItemListener(this);
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
        }
        
        @Override
        public void itemStateChanged(ItemEvent itemEvent) {
        }
        
        @Override
        public AccessibleStateSet getAccessibleStateSet() {
            AccessibleStateSet accessibleStateSet = super.getAccessibleStateSet();
            if (List.this.isMultipleMode()) {
                accessibleStateSet.add(AccessibleState.MULTISELECTABLE);
                return accessibleStateSet;
            }
            return accessibleStateSet;
        }
        
        @Override
        public AccessibleRole getAccessibleRole() {
            return AccessibleRole.LIST;
        }
        
        @Override
        public Accessible getAccessibleAt(Point point) {
            return null;
        }
        
        @Override
        public int getAccessibleChildrenCount() {
            return List.this.getItemCount();
        }
        
        @Override
        public Accessible getAccessibleChild(int n) {
            synchronized (List.this) {
                if (n >= List.this.getItemCount()) {
                    return null;
                }
                return new List.AccessibleAWTList.AccessibleAWTListChild(List.this, n);
            }
        }
        
        @Override
        public AccessibleSelection getAccessibleSelection() {
            return this;
        }
        
        @Override
        public int getAccessibleSelectionCount() {
            return List.this.getSelectedIndexes().length;
        }
        
        @Override
        public Accessible getAccessibleSelection(int n) {
            synchronized (List.this) {
                int accessibleSelectionCount = this.getAccessibleSelectionCount();
                if (n < 0 || n >= accessibleSelectionCount) {
                    return null;
                }
                return this.getAccessibleChild(List.this.getSelectedIndexes()[n]);
            }
        }
        
        @Override
        public boolean isAccessibleChildSelected(int n) {
            return List.this.isIndexSelected(n);
        }
        
        @Override
        public void addAccessibleSelection(int n) {
            List.this.select(n);
        }
        
        @Override
        public void removeAccessibleSelection(int n) {
            List.this.deselect(n);
        }
        
        @Override
        public void clearAccessibleSelection() {
            synchronized (List.this) {
                int[] selectedIndexes = List.this.getSelectedIndexes();
                if (selectedIndexes == null) {
                    return;
                }
                for (int i = selectedIndexes.length - 1; i >= 0; --i) {
                    List.this.deselect(selectedIndexes[i]);
                }
            }
        }
        
        @Override
        public void selectAllAccessibleSelection() {
            synchronized (List.this) {
                for (int i = List.this.getItemCount() - 1; i >= 0; --i) {
                    List.this.select(i);
                }
            }
        }
        
        protected class AccessibleAWTListChild extends Component.AccessibleAWTComponent implements Accessible
        {
            private static final long serialVersionUID = 4412022926028300317L;
            private List parent;
            private int indexInParent;
            
            public AccessibleAWTListChild(List parent, int indexInParent) {
                List.AccessibleAWTList.this.this$0.super();
                this.setAccessibleParent(this.parent = parent);
                this.indexInParent = indexInParent;
            }
            
            @Override
            public AccessibleContext getAccessibleContext() {
                return this;
            }
            
            @Override
            public AccessibleRole getAccessibleRole() {
                return AccessibleRole.LIST_ITEM;
            }
            
            @Override
            public AccessibleStateSet getAccessibleStateSet() {
                AccessibleStateSet accessibleStateSet = super.getAccessibleStateSet();
                if (this.parent.isIndexSelected(this.indexInParent)) {
                    accessibleStateSet.add(AccessibleState.SELECTED);
                    return accessibleStateSet;
                }
                return accessibleStateSet;
            }
            
            @Override
            public Locale getLocale() {
                return this.parent.getLocale();
            }
            
            @Override
            public int getAccessibleIndexInParent() {
                return this.indexInParent;
            }
            
            @Override
            public int getAccessibleChildrenCount() {
                return 0;
            }
            
            @Override
            public Accessible getAccessibleChild(int n) {
                return null;
            }
            
            @Override
            public Color getBackground() {
                return this.parent.getBackground();
            }
            
            @Override
            public void setBackground(Color background) {
                this.parent.setBackground(background);
            }
            
            @Override
            public Color getForeground() {
                return this.parent.getForeground();
            }
            
            @Override
            public void setForeground(Color foreground) {
                this.parent.setForeground(foreground);
            }
            
            @Override
            public Cursor getCursor() {
                return this.parent.getCursor();
            }
            
            @Override
            public void setCursor(Cursor cursor) {
                this.parent.setCursor(cursor);
            }
            
            @Override
            public Font getFont() {
                return this.parent.getFont();
            }
            
            @Override
            public void setFont(Font font) {
                this.parent.setFont(font);
            }
            
            @Override
            public FontMetrics getFontMetrics(Font font) {
                return this.parent.getFontMetrics(font);
            }
            
            @Override
            public boolean isEnabled() {
                return this.parent.isEnabled();
            }
            
            @Override
            public void setEnabled(boolean enabled) {
                this.parent.setEnabled(enabled);
            }
            
            @Override
            public boolean isVisible() {
                return false;
            }
            
            @Override
            public void setVisible(boolean visible) {
                this.parent.setVisible(visible);
            }
            
            @Override
            public boolean isShowing() {
                return false;
            }
            
            @Override
            public boolean contains(Point point) {
                return false;
            }
            
            @Override
            public Point getLocationOnScreen() {
                return null;
            }
            
            @Override
            public Point getLocation() {
                return null;
            }
            
            @Override
            public void setLocation(Point point) {
            }
            
            @Override
            public Rectangle getBounds() {
                return null;
            }
            
            @Override
            public void setBounds(Rectangle rectangle) {
            }
            
            @Override
            public Dimension getSize() {
                return null;
            }
            
            @Override
            public void setSize(Dimension dimension) {
            }
            
            @Override
            public Accessible getAccessibleAt(Point point) {
                return null;
            }
            
            @Override
            public boolean isFocusTraversable() {
                return false;
            }
            
            @Override
            public void requestFocus() {
            }
            
            @Override
            public void addFocusListener(FocusListener focusListener) {
            }
            
            @Override
            public void removeFocusListener(FocusListener focusListener) {
            }
        }
    }
}
